@@ -10,7 +10,7 @@
    This file is part of LibVEX, a library for dynamic binary
    instrumentation and translation.
 
-   Copyright (C) 2004-2005 OpenWorks LLP.  All rights reserved.
+   Copyright (C) 2004-2006 OpenWorks LLP.  All rights reserved.
 
    This library is made available under a dual licensing scheme.
 
@@ -491,10 +491,10 @@ typedef
             much as possible before continuing.  On SSE2 we emit a
             real "mfence", on SSE1 "sfence ; lock addl $0,0(%esp)" and
             on SSE0 "lock addl $0,0(%esp)".  This insn therefore
-            carries the subarch so the assembler knows what to
+            carries the host's hwcaps so the assembler knows what to
             emit. */
          struct {
-            VexSubArch subarch;
+            UInt hwcaps;
          } MFence;
 
          /* X86 Floating point (fake 3-operand, "flat reg file" insns) */
@@ -629,7 +629,7 @@ extern X86Instr* X86Instr_LoadEX    ( UChar szSmall, Bool syned,
 extern X86Instr* X86Instr_Store     ( UChar sz, HReg src, X86AMode* dst );
 extern X86Instr* X86Instr_Set32     ( X86CondCode cond, HReg dst );
 extern X86Instr* X86Instr_Bsfr32    ( Bool isFwds, HReg src, HReg dst );
-extern X86Instr* X86Instr_MFence    ( VexSubArch );
+extern X86Instr* X86Instr_MFence    ( UInt hwcaps );
 
 extern X86Instr* X86Instr_FpUnary   ( X86FpOp op, HReg src, HReg dst );
 extern X86Instr* X86Instr_FpBinary  ( X86FpOp op, HReg srcL, HReg srcR, HReg dst );
@@ -653,18 +653,19 @@ extern X86Instr* X86Instr_SseCMov   ( X86CondCode, HReg src, HReg dst );
 extern X86Instr* X86Instr_SseShuf   ( Int order, HReg src, HReg dst );
 
 
-extern void ppX86Instr ( X86Instr* );
+extern void ppX86Instr ( X86Instr*, Bool );
 
 /* Some functions that insulate the register allocator from details
    of the underlying instruction set. */
-extern void         getRegUsage_X86Instr ( HRegUsage*, X86Instr* );
-extern void         mapRegs_X86Instr     ( HRegRemap*, X86Instr* );
+extern void         getRegUsage_X86Instr ( HRegUsage*, X86Instr*, Bool );
+extern void         mapRegs_X86Instr     ( HRegRemap*, X86Instr*, Bool );
 extern Bool         isMove_X86Instr      ( X86Instr*, HReg*, HReg* );
-extern Int          emit_X86Instr        ( UChar* buf, Int nbuf, X86Instr* );
-extern X86Instr*    genSpill_X86         ( HReg rreg, Int offset );
-extern X86Instr*    genReload_X86        ( HReg rreg, Int offset );
+extern Int          emit_X86Instr        ( UChar* buf, Int nbuf, X86Instr*, 
+                                           Bool, void* dispatch );
+extern X86Instr*    genSpill_X86         ( HReg rreg, Int offset, Bool );
+extern X86Instr*    genReload_X86        ( HReg rreg, Int offset, Bool );
 extern void         getAllocableRegs_X86 ( Int*, HReg** );
-extern HInstrArray* iselBB_X86           ( IRBB*, VexArchInfo* );
+extern HInstrArray* iselBB_X86           ( IRBB*, VexArch, VexArchInfo* );
 
 #endif /* ndef __LIBVEX_HOST_X86_HDEFS_H */
 
