@@ -7,7 +7,7 @@
    This file is part of Valgrind, a dynamic binary instrumentation
    framework.
 
-   Copyright (C) 2000-2005 Julian Seward 
+   Copyright (C) 2000-2006 Julian Seward 
       jseward@acm.org
 
    This program is free software; you can redistribute it and/or
@@ -60,7 +60,7 @@
 
 /* After this many total errors have been observed, stop collecting
    errors at all.  Counterpart to M_COLLECT_NO_ERRORS_AFTER_SHOWN. */
-#define M_COLLECT_NO_ERRORS_AFTER_FOUND 100000
+#define M_COLLECT_NO_ERRORS_AFTER_FOUND 10000000
 
 /* The list of error contexts found, both suppressed and unsuppressed.
    Initially empty, and grows as errors are detected. */
@@ -447,9 +447,9 @@ void do_actions_on_error(Error* err, Bool allow_db_attach)
            && VG_(is_action_requested)( "Print suppression", &still_noisy ))
       ) {
       gen_suppression(err);
-      if (VG_(clo_gen_suppressions) == 1 && !still_noisy)
-         VG_(clo_gen_suppressions) = 0;
    }
+   if (VG_(clo_gen_suppressions) == 1 && !still_noisy)
+      VG_(clo_gen_suppressions) = 0;
 }
 
 /* Shared between VG_(maybe_record_error)() and VG_(unique_error)(),
@@ -767,7 +767,8 @@ void VG_(show_all_errors) ( void )
          StackTrace ips = VG_(extract_StackTrace)(p_min->where);
          VG_(translate) ( 0 /* dummy ThreadId; irrelevant due to debugging*/,
                           ips[0], /*debugging*/True, 0xFE/*verbosity*/,
-                          /*bbs_done*/0);
+                          /*bbs_done*/0,
+                          /*allow redir?*/True);
       }
 
       p_min->count = 1 << 30;
@@ -1109,12 +1110,17 @@ Bool supp_matches_callers(Error* err, Supp* su)
             break; 
 
          case FunName: 
-            // Nb: mangled names used in suppressions
-            if (!VG_(get_fnname_nodemangle)(a, caller_name, ERRTXT_LEN))
+            // Nb: mangled names used in suppressions.  Do, though,
+            // Z-demangle them, since otherwise it's possible to wind
+            // up comparing "malloc" in the suppression against
+            // "_vgrZU_libcZdsoZa_malloc" in the backtrace, and the
+            // two of them need to be made to match.
+            if (!VG_(get_fnname_Z_demangle_only)(a, caller_name, ERRTXT_LEN))
                VG_(strcpy)(caller_name, "???");
             break;
          default: VG_(tool_panic)("supp_matches_callers");
       }
+      if (0) VG_(printf)("cmp %s %s\n", su->callers[i].name, caller_name);
       if (!VG_(string_match)(su->callers[i].name, caller_name))
          return False;
    }
