@@ -7,7 +7,7 @@
    This file is part of Valgrind, a dynamic binary instrumentation
    framework.
 
-   Copyright (C) 2000-2005 Julian Seward
+   Copyright (C) 2000-2006 Julian Seward
       jseward@acm.org
 
    This program is free software; you can redistribute it and/or
@@ -90,6 +90,7 @@ extern Int   VG_(memcmp) ( const void* s1, const void* s2, SizeT n );
    ------------------------------------------------------------------ */
 
 // Check if an address/whatever is aligned
+#define VG_IS_2_ALIGNED(aaa_p)    (0 == (((Addr)(aaa_p)) & ((Addr)0x1)))
 #define VG_IS_4_ALIGNED(aaa_p)    (0 == (((Addr)(aaa_p)) & ((Addr)0x3)))
 #define VG_IS_8_ALIGNED(aaa_p)    (0 == (((Addr)(aaa_p)) & ((Addr)0x7)))
 #define VG_IS_16_ALIGNED(aaa_p)   (0 == (((Addr)(aaa_p)) & ((Addr)0xf)))
@@ -111,7 +112,7 @@ extern Int   VG_(memcmp) ( const void* s1, const void* s2, SizeT n );
 extern void VG_(ssort)( void* base, SizeT nmemb, SizeT size,
                         Int (*compar)(void*, void*) );
 
-/* Returns the base-2 logarithm of x. */
+/* Returns the base-2 logarithm of x.  Returns -1 if x is not a power of two. */
 extern Int VG_(log2) ( Int x );
 
 // A pseudo-random number generator returning a random UInt.  If pSeed
