@@ -7,7 +7,7 @@
    This file is part of Valgrind, a dynamic binary instrumentation
    framework.
 
-   Copyright (C) 2000-2005 Julian Seward 
+   Copyright (C) 2000-2006 Julian Seward 
       jseward@acm.org
 
    This program is free software; you can redistribute it and/or
@@ -70,6 +70,8 @@
 #  include "vki_posixtypes-amd64-linux.h"
 #elif defined(VGA_ppc32)
 #  include "vki_posixtypes-ppc32-linux.h"
+#elif defined(VGA_ppc64)
+#  include "vki_posixtypes-ppc64-linux.h"
 #else
 #  error Unknown platform
 #endif
@@ -150,6 +152,8 @@ typedef unsigned int	        vki_uint;
 #  include "vki-amd64-linux.h"
 #elif defined(VGA_ppc32)
 #  include "vki-ppc32-linux.h"
+#elif defined(VGA_ppc64)
+#  include "vki-ppc64-linux.h"
 #else
 #  error Unknown platform
 #endif
@@ -425,6 +429,12 @@ typedef struct vki_siginfo {
 #define VKI_BUS_ADRERR	(__VKI_SI_FAULT|2)	/* non-existant physical address */
 #define VKI_BUS_OBJERR	(__VKI_SI_FAULT|3)	/* object specific hardware error */
 
+/*
+ * SIGTRAP si_codes
+ */
+#define VKI_TRAP_BRKPT      (__VKI_SI_FAULT|1)  /* process breakpoint */
+#define VKI_TRAP_TRACE      (__VKI_SI_FAULT|2)  /* process trace trap */
+
 /*
  * This works because the alignment is ok on all current architectures
  * but we leave open this being overridden in the future
@@ -1089,6 +1099,7 @@ struct  vki_seminfo {
 //----------------------------------------------------------------------
 
 #define	VKI_ENOSYS		38	/* Function not implemented */
+#define	VKI_EOVERFLOW		75	/* Value too large for defined data type */
 
 //----------------------------------------------------------------------
 // From linux-2.6.8.1/include/linux/wait.h
@@ -1116,6 +1127,37 @@ struct  vki_seminfo {
 #define VKI_FUTEX_REQUEUE (3)
 #define VKI_FUTEX_CMP_REQUEUE (4)
 
+struct vki_robust_list {
+	struct vki_robust_list __user *next;
+};
+
+struct vki_robust_list_head {
+	/*
+	 * The head of the list. Points back to itself if empty:
+	 */
+	struct vki_robust_list list;
+
+	/*
+	 * This relative offset is set by user-space, it gives the kernel
+	 * the relative position of the futex field to examine. This way
+	 * we keep userspace flexible, to freely shape its data-structure,
+	 * without hardcoding any particular offset into the kernel:
+	 */
+	long futex_offset;
+
+	/*
+	 * The death of the thread may race with userspace setting
+	 * up a lock's links. So to handle this race, userspace first
+	 * sets this field to the address of the to-be-taken lock,
+	 * then does the lock acquire, and then adds itself to the
+	 * list, and then clears this field. Hence the kernel will
+	 * always have full knowledge of all locks that the thread
+	 * _might_ have taken. We check the owner TID in any case,
+	 * so only truly owned locks will be handled.
+	 */
+	struct vki_robust_list __user *list_op_pending;
+};
+
 //----------------------------------------------------------------------
 // From linux-2.6.8.1/include/linux/errno.h
 //----------------------------------------------------------------------
@@ -1610,6 +1652,8 @@ typedef struct vki_sg_scsi_id { /* used by SG_GET_SCSI_ID ioctl() */
                                            if available (struct cdrom_mcn) */
 #define VKI_CDROMVOLREAD	0x5313 /* Get the drive's volume setting
                                           (struct cdrom_volctrl) */
+#define VKI_CDROMREADRAW	0x5314	/* read data in raw mode (2352 Bytes)
+                                           (struct cdrom_read) */
 #define VKI_CDROM_CLEAR_OPTIONS	0x5321  /* Clear behavior options */
 #define VKI_CDROM_DRIVE_STATUS	0x5326  /* Get tray position, etc. */
 
@@ -1897,7 +1941,7 @@ struct vki_fb_var_screeninfo {
 };
 
 //----------------------------------------------------------------------
-// From linux-2.6.9/include/linux/kb.h
+// From linux-2.6.9/include/linux/kd.h
 //----------------------------------------------------------------------
 
 #define VKI_GIO_FONT       0x4B60  /* gets font in expanded form */
@@ -1942,8 +1986,21 @@ struct vki_consolefontdesc {
 #define VKI_PIO_UNISCRNMAP 0x4B6A  /* set full Unicode screen mapping */
 
 #define VKI_GIO_UNIMAP     0x4B66  /* get unicode-to-font mapping from kernel */
+struct vki_unipair {
+	unsigned short unicode;
+	unsigned short fontpos;
+};
+struct vki_unimapdesc {
+	unsigned short entry_ct;
+	struct vki_unipair __user *entries;
+};
 #define VKI_PIO_UNIMAP     0x4B67  /* put unicode-to-font mapping in kernel */
 #define VKI_PIO_UNIMAPCLR  0x4B68  /* clear table, possibly advise hash algorithm */
+struct vki_unimapinit {
+	unsigned short advised_hashsize;  /* 0 if no opinion */
+	unsigned short advised_hashstep;  /* 0 if no opinion */
+	unsigned short advised_hashlevel; /* 0 if no opinion */
+};
 
 #define VKI_KDGKBMODE      0x4B44  /* gets current keyboard mode */
 #define VKI_KDSKBMODE      0x4B45  /* sets current keyboard mode */
@@ -1997,6 +2054,19 @@ struct vki_kbd_repeat {
 
 #define VKI_KDFONTOP       0x4B72  /* font operations */
 
+struct vki_console_font_op {
+	unsigned int op;	/* operation code KD_FONT_OP_* */
+	unsigned int flags;	/* KD_FONT_FLAG_* */
+	unsigned int width, height;	/* font size */
+	unsigned int charcount;
+	unsigned char __user *data;	/* font data with height fixed to 32 */
+};
+
+#define VKI_KD_FONT_OP_SET		0	/* Set font */
+#define VKI_KD_FONT_OP_GET		1	/* Get font */
+#define VKI_KD_FONT_OP_SET_DEFAULT	2	/* Set font to default, data points to name / NULL */
+#define VKI_KD_FONT_OP_COPY		3	/* Copy from another console */
+
 //----------------------------------------------------------------------
 // From linux-2.6.9/include/linux/kb.h
 //----------------------------------------------------------------------
@@ -2053,6 +2123,55 @@ struct vki_serial_icounter_struct {
 
 #endif // __VKI_LINUX_H
 
+//----------------------------------------------------------------------
+// From linux-2.6.16/include/linux/vt.h
+//----------------------------------------------------------------------
+
+#define VKI_VT_OPENQRY	0x5600	/* find available vt */
+
+struct vki_vt_mode {
+	char mode;		/* vt mode */
+	char waitv;		/* if set, hang on writes if not active */
+	short relsig;		/* signal to raise on release req */
+	short acqsig;		/* signal to raise on acquisition */
+	short frsig;		/* unused (set to 0) */
+};
+#define VKI_VT_GETMODE	0x5601	/* get mode of active vt */
+#define VKI_VT_SETMODE	0x5602	/* set mode of active vt */
+
+struct vki_vt_stat {
+	unsigned short v_active;	/* active vt */
+	unsigned short v_signal;	/* signal to send */
+	unsigned short v_state;		/* vt bitmask */
+};
+#define VKI_VT_GETSTATE	0x5603	/* get global vt state info */
+#define VKI_VT_SENDSIG	0x5604	/* signal to send to bitmask of vts */
+
+#define VKI_VT_RELDISP	0x5605	/* release display */
+
+#define VKI_VT_ACTIVATE	0x5606	/* make vt active */
+#define VKI_VT_WAITACTIVE	0x5607	/* wait for vt active */
+#define VKI_VT_DISALLOCATE	0x5608  /* free memory associated to vt */
+
+struct vki_vt_sizes {
+	unsigned short v_rows;		/* number of rows */
+	unsigned short v_cols;		/* number of columns */
+	unsigned short v_scrollsize;	/* number of lines of scrollback */
+};
+#define VKI_VT_RESIZE	0x5609	/* set kernel's idea of screensize */
+
+struct vki_vt_consize {
+	unsigned short v_rows;	/* number of rows */
+	unsigned short v_cols;	/* number of columns */
+	unsigned short v_vlin;	/* number of pixel rows on screen */
+	unsigned short v_clin;	/* number of pixel rows per character */
+	unsigned short v_vcol;	/* number of pixel columns on screen */
+	unsigned short v_ccol;	/* number of pixel columns per character */
+};
+#define VKI_VT_RESIZEX      0x560A  /* set kernel's idea of screensize + more */
+#define VKI_VT_LOCKSWITCH   0x560B  /* disallow vt switching */
+#define VKI_VT_UNLOCKSWITCH 0x560C  /* allow vt switching */
+
 /*--------------------------------------------------------------------*/
 /*--- end                                                          ---*/
 /*--------------------------------------------------------------------*/
