@@ -11,7 +11,7 @@ FILE* OpenFile    PROTO((char *, char *));
 void  CommaPrint  PROTO((FILE *, intish));
 char *copystring  PROTO((char *));
 char *copystring2 PROTO((char *, char *));
-void *xmalloc	 PROTO((int));
-void *xrealloc	 PROTO((void *, int));
+void *xmalloc	 PROTO((size_t));
+void *xrealloc	 PROTO((void *, size_t));
 
 #endif /* UTILITIES_H */
